import logging
from ibmsecurity.utilities.tools import json_sort

logger = logging.getLogger(__name__)

uri = "/mga/scim/configuration"
requires_modules = ["mga", "federation"]
requires_version = "9.0.2"

def get(isamAppliance, check_mode=False, force=False):
    """
    Retrieving the complete list of SCIM configuration settings
    """
    return isamAppliance.invoke_get("Retrieving the complete list of SCIM configuration settings",
                                    "/mga/scim/configuration", requires_modules=requires_modules,
                                    requires_version=requires_version)


def set(isamAppliance, settings, check_mode=False, force=False):
    """
    Updating the complete SCIM configuration settings

    """
    obj = get(isamAppliance)
    obj = obj['data']
    sorted_new = json_sort(settings)
    sorted_old = json_sort(obj)
    if sorted_new == sorted_old:
        update_required = False
    else:
        update_required = True

    if force is True or update_required is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_put("Updating the complete SCIM configuration settings",
                                            "{0}".format(uri),
                                            settings,
                                            requires_modules=requires_modules,
                                            requires_version=requires_version
                                            )

    return isamAppliance.create_return_object(changed=False)



import logging
import ibmsecurity.utilities.tools

logger = logging.getLogger(__name__)


def get(isamAppliance, check_mode=False, force=False):
    """
    Get management ssl certificate information
    """
    return isamAppliance.invoke_get("Get management ssl certificate information",
                                    "/isam/management_ssl_certificate/")


def set(isamAppliance, certificate, password, check_mode=False, force=False):
    """
    Import certificate database
    """
    warnings = ["Idempotency not available. Unable to extract existing certificate to compare with provided one."]
    if force is True or _check(isamAppliance, certificate, password) is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True, warnings=warnings)
        else:
            return isamAppliance.invoke_post_files(
                "Import certificate database",
                "/isam/management_ssl_certificate",
                [
                    {
                        'file_formfield': 'cert',
                        'filename': certificate,
                        'mimetype': 'application/x-pkcs12'
                    }
                ],
                {
                    'password': password
                },
                json_response=False, warnings=warnings)

    return isamAppliance.create_return_object()


def _check(isamAppliance, certificate, password):
    """
    TODO replace placeholder
    requires additionally pyOpenSSL to load the p12 and extract the issuer, subject, etc
    """

    return False


def compare(isamAppliance1, isamAppliance2):
    """
    Compare management authentication settings
    """
    ret_obj1 = get(isamAppliance1)
    ret_obj2 = get(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import logging
import ibmsecurity.utilities.tools

logger = logging.getLogger(__name__)


def get_all(isamAppliance, check_mode=False, force=False):
    """
    Get management authorization - roles
    """
    return isamAppliance.invoke_get("Get management authorization - roles",
                                    "/authorization/roles/v1")


def get(isamAppliance, name, check_mode=False, force=False):
    """
    Get management authorization role
    """
    return isamAppliance.invoke_get("Get management authorization role",
                                    "/authorization/roles/{0}/v1".format(name))


def _check(isamAppliance, name):
    """
    Check if management authorization role exists
    """
    ret_obj = get_all(isamAppliance)

    for role in ret_obj['data']:
        if role['name'] == name:
            return True

    return False


def add(isamAppliance, name, check_mode=False, force=False):
    """
    Add a management authorization role
    """
    if force is True or _check(isamAppliance, name) is False:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_post("Add management authorization role",
                                             "/authorization/roles/v1",
                                             {
                                                 'name': name,
                                                 'users': None,
                                                 'groups': None,
                                                 'features': None
                                             })

    return isamAppliance.create_return_object()


def delete(isamAppliance, name, check_mode=False, force=False):
    """
    Delete a management authorization role
    """
    if force is True or _check(isamAppliance, name) is True:
        if check_mode is True:
            return isamAppliance.create_return_object(changed=True)
        else:
            return isamAppliance.invoke_delete(
                "Delete management authorization role",
                "/authorization/roles/{0}/v1".format(name))

    return isamAppliance.create_return_object()


def compare(isamAppliance1, isamAppliance2):
    """
    Compare all management authorization roles
    """
    ret_obj1 = get_all(isamAppliance1)
    ret_obj2 = get_all(isamAppliance2)

    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=[])



import logging

logger = logging.getLogger(__name__)


# *** WORK IN PROGRESS - module not done yet


def get_all(isamAppliance, isamUser, admin_domain='Default'):
    """
    Retrieve a list of POPs
    """
    ret_obj = isamAppliance.invoke_post("Retrieve a list of POPs",
                                        "/isam/pdadmin/poplistext/v1", {
                                            "admin_id": isamUser.username,
                                            "admin_pwd": isamUser.password,
                                            # "pop_name": pop_name,
                                            # "pop_attribute_name": pop_attribute_name,
                                            # "pop_attribute_value": pop_attribute_value,
                                            "admin_domain": admin_domain
                                        })
    ret_obj['changed'] = False

    return ret_obj


def get(isamAppliance, isamUser, pop_name, admin_domain='Default'):
    """
    Retrieve a specific POP
    """
    ret_obj = isamAppliance.invoke_post("Retrieve a specific POP",
                                        "/isam/pdadmin/popshowext/v1", {
                                            "admin_id": isamUser.username,
                                            "admin_pwd": isamUser.password,
                                            "pop_name": pop_name,
                                            "admin_domain": admin_domain
                                        })
    ret_obj['changed'] = False

    return ret_obj


def get_pop_list(isamAppliance, isamUser, object=None, pop_name=None, pop_attribute_name=None, pop_attribute_value=None,
                 admin_domain='Default'):
    """
    Retrieve a list of protected objects
    """
    ret_obj = isamAppliance.invoke_post("Retrieve a list of protected objects",
                                        "/isam/pdadmin/popfindext/v1", {
                                            "admin_id": isamUser.username,
                                            "admin_pwd": isamUser.password,
                                            "object": object,
                                            "pop_name": pop_name,
                                            "pop_attribute_name": pop_attribute_name,
                                            "pop_attribute_value": pop_attribute_value,
                                            "admin_domain": admin_domain
                                        })
    ret_obj['changed'] = False

    return ret_obj


def compare(isamAppliance1, isamAppliance2, isamUser, admin_domain='Default'):
    """
    Compare URL Mapping between two appliances
    """
    ret_obj1 = get_all(isamAppliance1, isamUser=isamUser, admin_domain=admin_domain)
    ret_obj2 = get_all(isamAppliance2, isamUser=isamUser, admin_domain=admin_domain)

    for obj in ret_obj1['data']:
        del obj['version']
        ret_obj = get(isamAppliance1, pop_name=obj['id'], isamUser=isamUser, admin_domain=admin_domain)
        obj['script'] = ret_obj['data']['contents']
    for obj in ret_obj2['data']:
        del obj['version']
        ret_obj = get(isamAppliance2, pop_name=obj['id'], isamUser=isamUser, admin_domain=admin_domain)
        obj['script'] = ret_obj['data']['contents']

    import ibmsecurity.utilities.tools
    return ibmsecurity.utilities.tools.json_compare(ret_obj1, ret_obj2, deleted_keys=['version'])
